import * as errors from "node_modules/errors.js/errors";
import * as response from "./response";

export type NewResponse = response.Response<APIResponse>;

export function New(data: any): NewResponse {
    if (!(data instanceof Object)) {
        return InvalidDataResp;
    }

    const ari = <apiResponse>data;
    if (!isAPIResponse(ari)) {
        return InvalidDataResp;
    }

    const code = <number>ari.code;
    const errors = !ari.errors ? null : <apiError[]>ari.errors;
    const success = ari.success === true;
    const ar = new APIResponse(code, ari.data, errors, success);
    return new response.Response<APIResponse>(ar, null);
}

export class APIResponse {
    code: number;
    data: any;
    errors: errors.ErrorList| null = null;
    success: boolean;

    constructor(code: number, data: any, errors: apiError[] | null, success: boolean) {
        this.code = code;
        this.data = data;
        this.success = success;

        if (!!errors) {
            this.setErrors(errors);
        }
    }

    private setErrors(errors: apiError[]) {
        errors.forEach((err: apiError) => this.pushError(err));
    }

    private pushError(error: apiError) {
        if (this.errors === null) {
            this.errors = new errors.ErrorList();
        }

        this.errors.Push(new APIError(error.message));
    }
}

interface apiResponse {
    code?: number;
    data?: any;
    errors?: apiError[];
    success?: boolean;
}

function isAPIResponse(ar: apiResponse): boolean {
    if (ar.code === undefined) {
        return false;
    }

    if (ar.success === undefined) {
        return false;
    }

    if (ar.data === undefined && ar.errors === undefined) {
        return false;
    }

    return true;
}

export class APIError {
    private message: string;

    constructor(message: string) {
        this.message = message;
    }

    Err() {
        return this.message;
    }
}

interface apiError {
    message: string;
}

export const NullResponse = new APIResponse(200, null, null, true);
export const ErrInvalidData = new errors.ErrorString("invalid data type, expected object");
export const InvalidDataResp = new response.Response<APIResponse>(NullResponse, ErrInvalidData);
