var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
define("node_modules/errors.js/errors", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ErrorList {
        constructor() {
            this.es = new Array();
        }
        handleError(err) {
            this.es.push(err.Err());
        }
        handleErrorList(el) {
            el.ForEach((err) => this.es.push(err));
        }
        handleErrorArray(ea) {
            ea.forEach((err) => this.handleError(err));
        }
        Err() {
            return this.es.join(", ");
        }
        ForEach(fn) {
            this.es.forEach(fn);
        }
        Push(err) {
            switch (err.constructor) {
                case ErrorList:
                    // Handle as an error list
                    return this.handleErrorList(err);
                case Array:
                    // Handle as an array of generic Errors
                    return this.handleErrorArray(err);
                default:
                    // Handle as generic Error
                    return this.handleError(err);
            }
        }
    }
    exports.ErrorList = ErrorList;
    class ErrorString {
        constructor(err) {
            this.err = err;
        }
        Err() {
            return this.err;
        }
    }
    exports.ErrorString = ErrorString;
});
define("response", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Response {
        constructor(value, err) {
            this.Value = value;
            this.Error = err;
        }
    }
    exports.Response = Response;
});
define("api", ["require", "exports", "node_modules/errors.js/errors", "response"], function (require, exports, errors, response) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    function New(data) {
        if (!(data instanceof Object)) {
            return exports.InvalidDataResp;
        }
        const ari = data;
        if (!isAPIResponse(ari)) {
            return exports.InvalidDataResp;
        }
        const code = ari.code;
        const errors = !ari.errors ? null : ari.errors;
        const success = ari.success === true;
        const ar = new APIResponse(code, ari.data, errors, success);
        return new response.Response(ar, null);
    }
    exports.New = New;
    class APIResponse {
        constructor(code, data, errors, success) {
            this.errors = null;
            this.code = code;
            this.data = data;
            this.success = success;
            if (!!errors) {
                this.setErrors(errors);
            }
        }
        setErrors(errors) {
            errors.forEach((err) => this.pushError(err));
        }
        pushError(error) {
            if (this.errors === null) {
                this.errors = new errors.ErrorList();
            }
            this.errors.Push(new APIError(error.message));
        }
    }
    exports.APIResponse = APIResponse;
    function isAPIResponse(ar) {
        if (ar.code === undefined) {
            return false;
        }
        if (ar.success === undefined) {
            return false;
        }
        if (ar.data === undefined && ar.errors === undefined) {
            return false;
        }
        return true;
    }
    class APIError {
        constructor(message) {
            this.message = message;
        }
        Err() {
            return this.message;
        }
    }
    exports.APIError = APIError;
    exports.NullResponse = new APIResponse(200, null, null, true);
    exports.ErrInvalidData = new errors.ErrorString("invalid data type, expected object");
    exports.InvalidDataResp = new response.Response(exports.NullResponse, exports.ErrInvalidData);
});
define("utilities", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var Types;
    (function (Types) {
        Types[Types["Text"] = 0] = "Text";
        Types[Types["JSON"] = 1] = "JSON";
        Types[Types["ArrayBuffer"] = 2] = "ArrayBuffer";
        Types[Types["Blob"] = 3] = "Blob";
        Types[Types["Document"] = 4] = "Document";
    })(Types = exports.Types || (exports.Types = {}));
    function GetRequestData(req, reqType, data) {
        if (data === undefined || data === null) {
            return null;
        }
        switch (reqType) {
            case Types.Text:
                req.setRequestHeader("Content-type", "text/plain");
                return data;
            case Types.JSON:
                req.setRequestHeader("Content-type", "application/json");
                return JSON.stringify(data);
            default:
                console.error('unknown request type', reqType);
                return null;
        }
    }
    exports.GetRequestData = GetRequestData;
    function GetURL(url) {
        var a = document.createElement("a");
        // This ensures that shortened urls are in fact a FQDN
        a.href = url;
        // Officially parses the URL
        var u = new URL(a.href);
        // Set timestamp search parameter as current unix time (in milliseconds)
        u.searchParams.set("ts", Date.now().toString());
        // Return new href
        return u.href;
    }
    exports.GetURL = GetURL;
});
define("http", ["require", "exports", "node_modules/errors.js/errors", "utilities", "api", "response"], function (require, exports, errors, utils, api, response) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.ErrEmptyURL = new errors.ErrorString("cannot use an empty url");
    exports.ErrInvalidReq = new errors.ErrorString("invalid request type provided");
    exports.EmptyURLResp = new response.Response(null, exports.ErrEmptyURL);
    exports.InvalidReqResp = new response.Response(null, exports.ErrInvalidReq);
    exports.DefaultResponseType = "text";
    const nullFn = (data) => { };
    function New(method, url, data, requestType, responseType) {
        return HttpRequest(method, url, data, requestType, responseType);
    }
    exports.New = New;
    function Get(url) {
        return HttpRequest("GET", url, null, utils.Types.JSON, utils.Types.JSON);
    }
    exports.Get = Get;
    function Post(url, data) {
        return HttpRequest("POST", url, data, utils.Types.JSON, utils.Types.JSON);
    }
    exports.Post = Post;
    function Put(url, data) {
        return HttpRequest("PUT", url, data, utils.Types.JSON, utils.Types.JSON);
    }
    exports.Put = Put;
    function Delete(url) {
        return HttpRequest("DELETE", url, null, utils.Types.JSON, utils.Types.JSON);
    }
    exports.Delete = Delete;
    function HttpRequest(method, url, data, requestType, responseType) {
        var fn = nullFn;
        var p = new Promise((resolve, reject) => {
            fn = resolve;
        });
        if (!url) {
            fn(exports.EmptyURLResp);
            return p;
        }
        if (!requestType) {
            fn(exports.InvalidReqResp);
            return p;
        }
        let req = new XMLHttpRequest();
        // Send cookies to CORs domains
        req.withCredentials = true;
        // Ensure method is all upper-case
        method = method.toUpperCase();
        // Prepare and send the request
        req.open(method, utils.GetURL(url), true);
        // Set response type
        req.responseType = getResponseType(responseType);
        // Set onload listener
        req.onload = handleLoad;
        // Send request
        req.send(utils.GetRequestData(req, requestType, data));
        // Return a promise
        return p;
        function handleLoad() {
            if (req.readyState !== 4) {
                return;
            }
            fn(getData(req, responseType));
        }
    }
    exports.HttpRequest = HttpRequest;
    function getData(req, rt) {
        if (rt == utils.Types.Text) {
            return new response.Response(req.responseText, getStatusCodeError(req));
        }
        // Process response
        const nar = api.New(req.response);
        if (nar.Error !== null) {
            // Return response generated directly from response value
            return new response.Response(req.response, getStatusCodeError(req));
        }
        // Assign api request const
        const ar = nar.Value;
        // Return response generated from api response
        return new response.Response(ar.data, ar.errors);
    }
    function getStatusCodeError(req) {
        if (req.status >= 200 && req.status < 400) {
            return null;
        }
        return new errors.ErrorString(req.statusText);
    }
    function getResponseType(rt) {
        return getResponseTypeStr(rt);
    }
    function getResponseTypeStr(rt) {
        switch (rt) {
            case utils.Types.JSON:
                return "json";
            case utils.Types.ArrayBuffer:
                return "arraybuffer";
            case utils.Types.Blob:
                return "blob";
            case utils.Types.Document:
                return "document";
            case utils.Types.Text:
                return "text";
            default:
                console.error("Non-supported response type, reverting to text");
                return "text";
        }
    }
});
define("jsonP", ["require", "exports", "node_modules/errors.js/errors", "response", "api"], function (require, exports, errors, response, api) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.ErrTimedOut = new errors.ErrorString("request timed out");
    exports.ErrLoading = new errors.ErrorString("error while loading");
    const prefix = "_jsonP_";
    const nullFn = function () { };
    function Get(url) {
        return __awaiter(this, void 0, void 0, function* () {
            return Request(url, null);
        });
    }
    exports.Get = Get;
    function Request(url, data) {
        return __awaiter(this, void 0, void 0, function* () {
            let fn;
            const p = new Promise((resolve, reject) => {
                fn = resolve;
            });
            // Set key
            const key = getKey();
            // Set timeout key
            const tok = setTimeout(() => error(exports.ErrTimedOut), 3000);
            // Create script
            const script = getScript(url, key, data);
            // Set error listener, because we do not have access to the response body nor the status code
            // during an unexpected error (EG 400, 404, 500), we have to return a non-descriptive ErrLoading
            script.onerror = () => error(exports.ErrLoading);
            // Append script to document body
            document.body.appendChild(script);
            // Set window function
            window[key] = success;
            // Return promise
            return p;
            function success(data) {
                exit();
                fn(getResponse(data));
            }
            function error(err) {
                exit();
                fn(new response.Response(null, err));
            }
            function exit() {
                clearTimeout(tok);
                removeScript(script);
                script.onerror = nullFn;
            }
        });
    }
    exports.Request = Request;
    function getScript(src, key, payload) {
        const script = document.createElement("script");
        script.src = src;
        script.type = "application/javascript";
        const u = new URL(script.src);
        u.searchParams.set("callback", key);
        if (payload !== null) {
            u.searchParams.set("data", encodeURI(JSON.stringify(payload)));
        }
        script.src = u.href;
        return script;
    }
    function getKey() {
        return prefix + Date.now().toString() + getRandom(100000, 999999).toString();
    }
    function getRandom(min, max) {
        return Math.round(Math.random() * (max - min) + min);
    }
    function getResponse(data) {
        const nar = api.New(data);
        if (nar.Error !== null) {
            return new response.Response(data, null);
        }
        return new response.Response(nar.Value.data, nar.Value.errors);
    }
    function removeScript(script) {
        let parent = script.parentElement;
        if (!parent) {
            return;
        }
        parent.removeChild(script);
    }
});
