import * as errors from "node_modules/errors.js/errors"

export enum Types {
	Text = 0,
	JSON,
	ArrayBuffer,
	Blob,
	Document,
}

export function GetRequestData(req: XMLHttpRequest, reqType: Types, data: any): string | null {
	if (data === undefined || data === null) {
		return null;
	}

	switch (reqType) {
		case Types.Text:
			req.setRequestHeader("Content-type", "text/plain");
			return data;

		case Types.JSON:
			req.setRequestHeader("Content-type", "application/json");
			return JSON.stringify(data);

		default:
			console.error('unknown request type', reqType);
			return null;
	}
}

export function GetURL(url: string) {
	var a = document.createElement("a");
	// This ensures that shortened urls are in fact a FQDN
	a.href = url;
	// Officially parses the URL
	var u = new URL(a.href);
	// Set timestamp search parameter as current unix time (in milliseconds)
	u.searchParams.set("ts", Date.now().toString());
	// Return new href
	return u.href;
}
